function imT = translateIm(im, xVec, interpType)
%  imT = translateIm(im, xVec, interpType)
%
% Translate a 2D image im by the 2-vector xVec = (x,y) (possibly real-valued).
% Input: 
%   im - a grey-level image represented by  n x m double array, 
%   xVec - a 2-vector (type double) specifying the translation,
%   interpType - either 'bilinear' or 'bicubic'
% Return imT having the same size as im, with:
%   imT(i,j) apprx= im(i-xVec(2), j-xVec(1)) 
% when 1<=i-xVec(2)<=size(im,1) and 1<=j-xVec(1)<=size(im,2).  Otherwise
% the pixel imT(i,j) falls outside of the boundary of im and its value
% is set to nan. Triangular (1D) filter kernels are used when
% interpType is 'bilinear', and the 1D  Catmull-Rom filter kernels are
% used when interpType is 'bicubic'.

  if nargin < 3
    interpType = 'bilinear';
  end
  
  %% Compute a discrete and residual shift
  sVec = floor(xVec);
  dxVec = xVec - sVec;  % both components are >= 0.
  
  %% Since the convolution kernels have even length,
  %% and are not symmetric, dx = 1 is preferred over dx = 0.
  idx = dxVec == 0;
  if any(idx)
    dxVec(idx) = 1;
    sVec(idx) = sVec(idx)-1;
  end
  
  imT = [];
  %% Form the filter kernels for the residual shifts dxVec > 0
  if strcmp(interpType, 'bilinear')
    fx = [1-dxVec(1), dxVec(1)];
    fy = [1-dxVec(2), dxVec(2)];
  elseif strcmp(interpType, 'bicubic')
    fx = catmullRomFilter(dxVec(1));
    fy = catmullRomFilter(dxVec(2));
  else
    fprintf(2,'Unknown interp type %s\n', interpType);
    return;
  end  
  
  tmp = rconv2sep(im, fx, fy);
  %% Since the filter kernels have even length, the
  %% convolved response is shifted by [1,1] (up and to the left).
  
  imT = zeros(size(tmp));
  % We want 
  % imT = im(y-xVec(2), x-xVec(1)) 
  %     = im(y-sVec(2)-dxVec(2), x-sVec(1) -dxVec(1)) 
  [x, y] = meshgrid(1:size(im,2), 1:size(im,1));
  idIm = (x - sVec(1)-1>0) & (x - sVec(1)-1 <= size(im,2));
  idIm = idIm & (y - sVec(2)-1>0) & (y -sVec(2)-1 <= size(im,1));
  imT(idIm) = tmp( y(idIm) - sVec(2)-1 + size(imT,1)*( x(idIm)-sVec(1)-2));
  imT(~idIm) = nan;
  
  return;

  %%%%%%%%%%%%%%%%% Test image translate

  %% Bilinear test (linear image)
  im = zeros([100 100]);
  szIm = [100 100]
  [x y] = meshgrid(1:szIm(2), 1:szIm(1));
  a = [1 1];
  im = a(1)*x+a(2)*y;
  xVec = [1.3 -3.7];
  
  imT = translateIm(im, xVec, 'bilinear');
  
  % imT(j,i) = im(j-xVec(2), i-xVec(1)) 
  %  = a(2)*x(j-xVec(2),i-xVec(1)) + a(1)*y(j-xVec(2),i-xVec(1)) 
  %  = a(2)*(j-xVec(2)) + a(1)* (i-xVec(1)))
  % im(j,i) = a(2)*j + a(1)*i
  % imT(j,i) - im(j,i) = -a(2)*xVec(2) - a(1)*xVec(1)
  
  figure(1); clf;
  showIm(double(isnan(imT)));
  pause;
  
  errIm = imT - im + a * xVec';
  errIm(isnan(imT)) = 0;
  figure(1); clf;
  subplot(1,2,1);
  showIm(errIm);
  imstats(errIm);
  subplot(1,2,2);
  % away from boundaries
  showIm(errIm(10:end-10,10:end-10));
  % Error should be zero away from boundaries
  imstats(errIm(10:end-10,10:end-10));
  pause;
  
 %% Bilinear test (quadratic image)
  im = zeros([100 100]);
  szIm = [100 100]
  [x y] = meshgrid(1:szIm(2), 1:szIm(1));
  a = [1 1 0.01 -0.02 0.01];
  im = a(1)*x+a(2)*y + a(3) * x.*x +  a(4) * x.*y  +  a(5) * y.*y ;
  figure(1);clf; showIm(im);
  pause;
  xVec = [1.3 -3.7];

  imT = translateIm(im, xVec, 'bilinear');
  
  % imT(j,i) = im(j-xVec(2), i-xVec(1)) 
  figure(1); clf;
  showIm(double(isnan(imT)));
  pause;
  
  xs = x - xVec(1); ys = y - xVec(2);
  im0 = a(1)*xs+a(2)*ys + a(3) * xs.*xs +  a(4) * xs.*ys  +  a(5) * ys.*ys ;
 
  errIm = imT - im0;
  errIm(isnan(imT)) = 0;
  figure(1); clf;
  subplot(1,2,1);
  showIm(errIm);
  imstats(errIm);
  subplot(1,2,2);
  % away from boundaries
  showIm(errIm(10:end-10,10:end-10));
  
  % Error should be small but non-zero (0.0042) away from boundaries
  imstats(errIm(10:end-10,10:end-10));
  pause;
  
  %%%%%%%%%%%%%% Bicubic test (linear image)
  im = zeros([100 100]);
  szIm = [100 100]
  [x y] = meshgrid(1:szIm(2), 1:szIm(1));
  a = [1 1];
  im = a(1)*x+a(2)*y;
  xVec = [1.3 -3.7];

  imT = translateIm(im, xVec, 'bicubic');
  
  % imT(j,i) = im(j-xVec(2), i-xVec(1)) 
  %  = a(2)*x(j-xVec(2),i-xVec(1)) + a(1)*y(j-xVec(2),i-xVec(1)) 
  %  = a(2)*(j-xVec(2)) + a(1)* (i-xVec(1)))
  % im(j,i) = a(2)*j + a(1)*i
  % imT(j,i) - im(j,i) = -a(2)*xVec(2) - a(1)*xVec(1)
  
  figure(1); clf;
  showIm(double(isnan(imT)));
  pause;
  
  errIm = imT - im + a * xVec';
  errIm(isnan(imT)) = 0;
  figure(1); clf;
  subplot(1,2,1);
  showIm(errIm);
  imstats(errIm);
  subplot(1,2,2);
  % away from boundaries
  showIm(errIm(10:end-10,10:end-10));
  % Error should be zero away from boundaries
  imstats(errIm(10:end-10,10:end-10));
  pause;

 %% Bicubic test (quadratic image)
  im = zeros([100 100]);
  szIm = [100 100]
  [x y] = meshgrid(1:szIm(2), 1:szIm(1));
  a = [1 1 0.01 -0.02 0.01];
  im = a(1)*x+a(2)*y + a(3) * x.*x +  a(4) * x.*y  +  a(5) * y.*y ;
  figure(1);clf; showIm(im);
  pause;
  xVec = [1.3 -3.7];

  imT = translateIm(im, xVec, 'bicubic');
  
  % imT(j,i) = im(j-xVec(2), i-xVec(1)) 
  figure(1); clf;
  showIm(double(isnan(imT)));
  pause;
  
  xs = x - xVec(1); ys = y - xVec(2);
  im0 = a(1)*xs+a(2)*ys + a(3) * xs.*xs +  a(4) * xs.*ys  +  a(5) * ys.*ys ;
 
  errIm = imT - im0;
  errIm(isnan(imT)) = 0;
  figure(1); clf;
  subplot(1,2,1);
  showIm(errIm);
  imstats(errIm);
  subplot(1,2,2);
  % away from boundaries
  showIm(errIm(10:end-10,10:end-10));
  
  % Error should be zero away from boundaries
  imstats(errIm(10:end-10,10:end-10));
  pause;

    
  